package com.datacloudsec.utils;

import com.datacloudsec.utils.BASE64Util;
import java.io.IOException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import javax.crypto.Cipher;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

public class RsaAndDsa {

   private String keyAlgorithm;
   private String signatureAlgorithm;
   private KeyPair keyPair;


   public RsaAndDsa(String keyAlgorithm, String signatureAlgorithm) {
      this.keyAlgorithm = keyAlgorithm;
      this.signatureAlgorithm = signatureAlgorithm;
   }

   public void generateKeyPair(int size) throws NoSuchAlgorithmException {
      KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance(this.keyAlgorithm);
      keyPairGen.initialize(size);
      this.keyPair = keyPairGen.generateKeyPair();
   }

   public PublicKey getPublicKey() throws Exception {
      if(this.keyPair == null) {
         throw new Exception("未生成密钥对");
      } else {
         return this.keyPair.getPublic();
      }
   }

   public String getPublicKeyString() throws Exception {
      return encryptBASE64(this.getPublicKey().getEncoded());
   }

   public PrivateKey getPrivateKey() throws Exception {
      if(this.keyPair == null) {
         throw new Exception("未生成密钥对");
      } else {
         return this.keyPair.getPrivate();
      }
   }

   public String getPrivateKeyString() throws Exception {
      return encryptBASE64(this.getPrivateKey().getEncoded());
   }

   public byte[] encryptByPrivateKey(byte[] data, String key) throws Exception {
      byte[] keyBytes = decryptBASE64(key);
      PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance(this.keyAlgorithm);
      PrivateKey privateKey = keyFactory.generatePrivate(pkcs8KeySpec);
      Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
      cipher.init(1, privateKey);
      return cipher.doFinal(data);
   }

   public byte[] encryptByPublicKey(byte[] data, String key) throws Exception {
      byte[] keyBytes = decryptBASE64(key);
      X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance(this.keyAlgorithm);
      PublicKey publicKey = keyFactory.generatePublic(x509KeySpec);
      Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
      cipher.init(1, publicKey);
      return cipher.doFinal(data);
   }

   public byte[] decryptByPrivateKey(byte[] data, String key) throws Exception {
      byte[] keyBytes = decryptBASE64(key);
      PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance(this.keyAlgorithm);
      PrivateKey privateKey = keyFactory.generatePrivate(pkcs8KeySpec);
      Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
      cipher.init(2, privateKey);
      return cipher.doFinal(data);
   }

   public byte[] decryptByPublicKey(byte[] data, String key) throws Exception {
      byte[] keyBytes = decryptBASE64(key);
      X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance(this.keyAlgorithm);
      PublicKey publicKey = keyFactory.generatePublic(x509KeySpec);
      Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
      cipher.init(2, publicKey);
      return cipher.doFinal(data);
   }

   public String sign(byte[] data, String privateKey) throws Exception {
      byte[] keyBytes = decryptBASE64(privateKey);
      PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance(this.keyAlgorithm);
      PrivateKey priKey = keyFactory.generatePrivate(pkcs8KeySpec);
      Signature signature = Signature.getInstance(this.signatureAlgorithm);
      signature.initSign(priKey);
      signature.update(data);
      return encryptBASE64(signature.sign());
   }

   public boolean verify(byte[] data, String publicKey, String sign) throws Exception {
      byte[] keyBytes = decryptBASE64(publicKey);
      X509EncodedKeySpec keySpec = new X509EncodedKeySpec(keyBytes);
      KeyFactory keyFactory = KeyFactory.getInstance(this.keyAlgorithm);
      PublicKey pubKey = keyFactory.generatePublic(keySpec);
      Signature signature = Signature.getInstance(this.signatureAlgorithm);
      signature.initVerify(pubKey);
      signature.update(data);
      return signature.verify(decryptBASE64(sign));
   }

   private static String encryptBASE64(byte[] content) {
      return (new BASE64Encoder()).encodeBuffer(content);
   }

   private static byte[] decryptBASE64(String key) throws IOException {
      return (new BASE64Decoder()).decodeBuffer(key);
   }

   public static void main(String[] args) throws Exception {
      RsaAndDsa r = new RsaAndDsa("RSA", "MD5withRSA");
      r.generateKeyPair(1024);
      String data = "123456";
      String publicKey = r.getPublicKeyString();
      String privateKey = r.getPrivateKeyString();
      System.out.println("公钥：" + publicKey);
      System.out.println("私钥：" + privateKey);
      byte[] pass = r.encryptByPrivateKey(data.getBytes(), privateKey);
      System.out.println("私钥加密BASE64: enc:" + encryptBASE64(pass));
      String base64 = BASE64Util.encodeByte(pass);
      System.out.println("解密数据：" + new String(r.decryptByPublicKey(decryptBASE64(base64), publicKey)));
      String signStr = r.sign(data.getBytes(), privateKey);
      System.out.println(r.verify(data.getBytes(), publicKey, signStr));
   }
}
